---- KordaCooldown
-- Written by Korda @ Tarren Mill EU / henrik_s @ wowinterface.com

-- test

---- Configuration
-- Set which spells to monitor. Spells that share names should be referenced by spell ID.
-- Pet spells and abilities can be added as well.
local spells = {
	["PRIEST"] = {
		"Mind Blast", 
		"Dispersion", 
		"Shadow Word: Death", 
		"Shadowfiend", 
		"Prayer of Mending",
		"Chakra",
		"Circle of Healing",
		"Desperate Prayer",
		"Guardian Spirit",
		88625, -- Holy Word: Chastise
		88684, -- Holy Word: Serenity
		88685, -- Holy Word: Sanctuary
		"Fade"
	},
	
	["SHAMAN"] = {
		"Stormstrike", 
		"Lava Lash", 
		"Earth Shock", 
		"Unleash Elements",
		"Lava Burst",
		"Flame Shock",
		"Fire Nova",
		"Frost Shock",
		"Wind Shear",
		"Hex",
		"Feral Spirit"
	},
	
	["ROGUE"] = { },
	
	["DRUID"] = {
		"Tiger's Fury",
		"Berserk",
		"Mangle",
		"Enrage",
		779, -- Bear swipe
		49376, -- Cat charge
		16979, -- Bear charge
		"Maul",
		"Faerie Fire (Feral)",
		"Survival Instincts",
		"Barkskin",
		"Growl",
		"Bash",
		"Frenzied Regeneration",
		"Cower",
		"Tree of Life",
		"Swiftmend",
		"Nature's Swiftness",
		"Wild Growth",
		"Thrash"
	},
	
	["MAGE"] = { },
	
	["WARLOCK"] = {
		"Chaos Bolt"
	},
	
	["PALADIN"] = { },
	
	["DEATHKNIGHT"] = { },
	
	["HUNTER"] = {
		"Explosive Shot", 
		"Raptor Strike", 
		"Kill Command", 
		"Kill Shot",
		"Intimidation",
		"Fervor",
		
		-- Pet
		"Dive",
		"Dash",
		"Charge",
	},
	
	["WARRIOR"] = {
		"Charge", 
		"Thunder Clap",
		"Mortal Strike", 
		"Heroic Strike", 
		"Taunt", 
		"Shield Slam",
		"Shield Bash", 
		"Sweeping Strikes",
		"Last Stand", 
		"Shield Block", 
		"Revenge"
	}
}

-- Timer update throttle (in seconds).
local throttle = 0.1

-- Position (pixels)
local x, y = 0, 120
----

local SetPoint = SetPoint
local GetTime = GetTime
local SetWidth = SetWidth
local pairs = pairs

local frames = { }

local castSpell, castSpellID

spells = spells[select(2, UnitClass("player"))]

local width, height = 44, 48
local barwidth = 40
local spacing = 2

local anchor = CreateFrame("Frame", nil, UIParent)
anchor:SetPoint("BOTTOM", UIParent, "BOTTOM", x, y)
anchor:SetHeight(height)

local right = anchor:CreateTexture()
right:SetHeight(height + 4)
right:SetWidth(2)
right:SetTexture(0, 0, 0, 0.75)
right:SetPoint("LEFT", anchor, "RIGHT")

local function updatePosition()
	local shown = 0
	for k,_ in pairs(frames) do
		local frame = frames[k]
		if frame:IsVisible() then
			shown = shown + 1
			frame:SetPoint("LEFT", anchor, "LEFT", shown == 1 and 0 or (((shown - 1) * width) + (spacing * (shown - 1))), -2)
		end
	end
	
	if shown > 0 then
		anchor:Show()
		anchor:SetWidth((shown * width) + ((shown - 1) * spacing))
	else
		anchor:Hide()
	end
end

local function updateSpell(self)
	local start, duration = GetSpellCooldown(self.spell)
	if duration and duration > 1.5 and not self:IsVisible() and (castSpell == self.spell or castSpellID == self.spell) then
		self.start, self.duration = start, duration
		
		self.rem = self.start - GetTime() + self.duration
		self.bar:SetWidth((self.rem / self.duration) * barwidth)

		self:Show()

		updatePosition()
	end
end

local function updateTimer(self, elapsed)
	self.lastupdate = self.lastupdate + elapsed
	if self.lastupdate > throttle then
		local _, duration = GetSpellCooldown(self.spell)
		self.rem = self.start - GetTime() + self.duration
		self.bar:SetWidth((self.rem / self.duration) * barwidth)
		
		if duration == 0 or self.rem < 0 then
			self:Hide()
			updatePosition()
		end

		self.lastupdate = 0
	end
end

local function createIcon(spell)
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetHeight(height)
	frame:SetWidth(width)
	
	frame.spell = spell

	local icon = frame:CreateTexture()
	icon:SetPoint("CENTER", frame, "CENTER")
	icon:SetTexture(GetSpellTexture(spell))
	icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	icon:SetHeight(38)
	icon:SetWidth(38)

	local border = frame:CreateTexture()
	border:SetPoint("CENTER", frame, "CENTER")
	border:SetTexture("Interface\\AddOns\\KordaCooldown\\border")
	border:SetHeight(128)
	border:SetWidth(128)

	local bar = frame:CreateTexture(nil, "OVERLAY")
	bar:SetPoint("RIGHT", frame, "TOPRIGHT", -2, -1)
	bar:SetTexture(1, 1, 1)
	bar:SetHeight(2)
	bar:SetWidth(barwidth)
	frame.bar = bar

	frame.lastupdate = 0
	frame:SetScript("OnUpdate", updateTimer)

	frame:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	frame:SetScript("OnEvent", updateSpell)
	
	frame:Hide()

	return frame
end

local function load(self, event, ...)
	if event == "SPELLS_CHANGED" or event == "PLAYER_ENTERING_WORLD" or event == "UNIT_PET" then
		for k, v in pairs(spells) do
			if GetSpellInfo(v) then
				frames[v] = frames[v] or createIcon(spells[k])
			end
		end
	end
	
	if event == "UNIT_SPELLCAST_SUCCEEDED" and (... == "player" or ... == "pet") then
		_, castSpell, _, _, castSpellID = ...
	end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
frame:RegisterEvent("SPELLS_CHANGED")
frame:RegisterEvent("UNIT_PET")

frame:SetScript("OnEvent", load)